/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    RemoveRedundancy.java
 *    Copyright (C) 2008 Yasser EL-Manzalawy (yasser at cs.iastate.edu)
 *
 */


/*
 *    The input is a dataset of peptides (in fasta format).
 *    The output is a similarity-reduced set of peptides (in fasta format).
 *    Two peptides are similar if they share at least <cutoff>% value.
 * 	This programs required the installation of EMBOSSWin package.
 */
/*
 * This programs has been used in [1] to derive SRDS2 out of SRDS1 datasets.
 * [1] EL-Manzalawy, Y., Dobbs, D., Honavar, V., 2008. On evaluating MHC-II binding peptide prediction methods. Submitted to PLoS ONE.
 */


package evalscripts;
import java.io.*;
import java.util.*;
import weka.core.*;

public class RemoveRedundancy {

	public static void main(String[] args) throws Exception{
		
		if (args.length < 3){
			System.out.println("Usage: RemoveRedundancy <inFile> <outFile> <cutoff>");
			System.out.println("inFile and outFile are in fasta format");
			System.out.println("cutoff in percents (e.g., 80)");
			System.exit(0);
		}
		
		String inFile = args[0];
		String outFile = args[1];
		double thrd = Double.parseDouble(args[2]);
		
		RemoveRedundancy rr = new RemoveRedundancy();
		rr.getNRDataset(inFile, outFile, thrd);		
	}

	/***
	 * Aligns two sequences and return the percent identity (%)
	 * @param seq1
	 * @param seq2
	 * @return
	 * @throws Exception
	 */
	public double align(String seq1, String seq2) throws Exception{
		String tmp_dir = System.getProperty("java.io.tmpdir");	//get system tmp directory
		
		double id = 0;
		// write each sequence into a fasta file
		String file1 = tmp_dir+"s1.fasta";
		String file2 = tmp_dir+"s2.fasta";
		String outfile = tmp_dir+"algn.fasta";
		
		Writer w = new BufferedWriter (new FileWriter (file1));
		w.write(">s1 \n");
		w.write(seq1);
		w.close();
		
		w = new BufferedWriter (new FileWriter (file2));
		w.write(">s2 \n");
		w.write(seq2);
		w.close();
		
		// align
		try{
			Process p = null;			
			p = Runtime.getRuntime().exec("C:/EMBOSSwin/needle "+ file1 +" "+ file2+" -gapopen 10 -gapextend 0.5 -datafile EBLOSUM62 -outfile "+ tmp_dir+"algn.fasta" );
			int exitVal = p.waitFor();
		}catch (Exception e){
			System.out.println("align: Error aligning two sequences");
			e.printStackTrace();
	        System.out.println(e.getMessage());
		}
		//parse output file
		File content1 = new File (outfile);
		InputStreamReader is1 =  new InputStreamReader (new FileInputStream (content1));
		BufferedReader reader1 = new BufferedReader (is1);
		String line = null;
		while((line=reader1.readLine())!= null){			
			if(line.indexOf("Similarity:") > 0)				
				break;
		}
		id = Double.valueOf(line.substring(line.indexOf("(")+1, line.indexOf(")")-1)).doubleValue();
		return id;		
	}

	/**
	 * 
	 * @param dataFile	input file in fasta format
	 * @param outFile	output file in fasta format
	 * @param thrd      similarity cutoff in (%)
	 * @throws Exception
	 */
	public void getNRDataset (String dataFile, String outFile, double thrd) throws Exception {
		Vector sequences = loadSequences (dataFile);
		int n = sequences.size();
		int[][] S = new int [n][n];		// similarity matrix
		int[] score = new int [n];		// score[i] = no of sequences that sequence i is similar to them
		int[] nr_set = new int[n];		// indexes of nr sequences
		int nr_count = 0;				// size of nr set
		
		for(int row = 0; row < n; row++){
			for (int col = row; col < n; col++){
				if (row==col)
					S[row][col] = 1;	//diagonal = 1
				else{
					if (align ((String)sequences.elementAt(row), (String)sequences.elementAt(col)) >= thrd){
						S[row][col] = 1;
						S[col][row] = 1;
					}
					else{
						S[row][col] = 0;
						S[col][row] = 0;
					}
				}
			}// end col
		}//end row
		
	
		// compute the score matrix
		for(int row = 0; row < n; row++){
			for (int col = 0; col < n; col++){
				if (S[row][col]==1)
					score[row]++;
			}
		}
		
		
		int index = -1;
		do {
			index = Utils.minIndex(score);
			if (score[index]< (n+1)){
				nr_set[nr_count++]= index;
				score[index] = n+1;
				for (int col = 0; col < n; col++){
					if (S[index][col]==1)
						score[col] = n+1;
				}				
			}
			else
				break;				
		} while (true);
		
		Writer w = new BufferedWriter (new FileWriter (outFile));
		
		for (int i = 1; i <= nr_count; i++){
			w.write(">" + i + "\n");
			w.write((String)sequences.elementAt(nr_set[i-1])+"\n");
		}
		w.close();
		System.out.println("size of " + dataFile + " is " + n);
		System.out.println("size of nr set is " + nr_count);
	}
	
	/**
	 * Loads peptides in fasta format
	 * @param dataFile fasta input file
	 * @return Vector of peptide sequences
	 * @throws Exception
	 */
	public Vector loadSequences (String dataFile) throws Exception {
		Vector seqs = new Vector();
		
		File content1 = new File (dataFile);
		InputStreamReader is1 =  new InputStreamReader (new FileInputStream (content1));
		BufferedReader reader1 = new BufferedReader (is1);
		String line = null;
		String s = null;		
		while((line=reader1.readLine())!= null){
			if (line.startsWith(">")){
				if (s == null)	//first seq
					s = new String();
				else{
					seqs.add(s);
					s = new String();
				}
			}
			else{
				s += line;
			}
		}
		seqs.add(s);		
		return seqs;
	}
}
