/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    RemoveRedundancyBench.java
 *    Copyright (C) 2008 Yasser EL-Manzalawy (yasser at cs.iastate.edu)
 *
 */


/*
 *    The input is a dataset of peptides (in fasta format).
 *    The output is a similarity-reduced set of peptides (in fasta format).
 *    Two peptides are similar if their PID is at least <pid>
 */
/*
 * This programs has been used in [1] to derive SRDS3 out of UPDS datasets.
 * [1] EL-Manzalawy, Y., Dobbs, D., Honavar, V., 2008. On evaluating MHC-II binding peptide prediction methods. Submitted to PLoS ONE.
 */

package evalscripts;

import java.io.*;
import java.util.*;
import weka.core.*;
public class RemoveRedundancyBench {

	public static void main(String[] args) throws Exception{
		
		if (args.length < 3){
			System.out.println("Usage: RemoveRedundancyBench <inFile> <outFile> <pid>");
			System.out.println("inFile and outFile are in fasta format");
			System.out.println("percent identity (pid) (e.g., 0.8)");
			System.exit(0);
		}
		
		String inFile = args[0];
		String outFile = args[1];
		double pid = Double.parseDouble(args[2]);

		RemoveRedundancyBench rr = new RemoveRedundancyBench();
		rr.getNRDataset(inFile, outFile, pid);
	}
	/***
	 * Two peptides will be similar if they share nonamer 
	 * @param seq1
	 * @param seq2
	 * @return 1 -> similar , otherwise 0
	 * @throws Exception
	 */
	public double align(String seq1, String seq2, double pid) throws Exception{
		int slen;
		int llen;
		char[] s;
		String lseq;
		
		if (seq1.length() <= seq2.length()){
			slen = seq1.length();
			llen = seq2.length();
			s = seq1.toCharArray();
			lseq = seq2;
		}
		else{
			slen = seq2.length();
			llen = seq1.length();
			s = seq2.toCharArray();
			lseq = seq1;
		}

		for (int start = 0; start <= (llen-slen); start++){				
			int end = start +  slen;
			String seq = lseq.substring(start,end);
			char[] ch = seq.toCharArray();
			double count = 0;
			for (int k = 0; k < slen; k++){
				if (s[k]==ch[k])
					count++;
			}
			count = (double) (count/slen);			
			if (count >= pid){				
				return 1;
			}
		}		
		return 0;
	}
	/*****
	 * 
	 * @param dataFile sequences in fasta format
	 * @throws Exception
	 */
	public void getNRDataset (String dataFile, String outFile, double thrd) throws Exception {
		Vector sequences = loadSequences (dataFile);
		int n = sequences.size();
		int[][] S = new int [n][n];		//similarity matrix
		int[] score = new int [n];		//score[i] = no of sequences that sequence i is similar to them
		int[] nr_set = new int[n];		// indexes of nr sequences
		int nr_count = 0;				// size of nr set
		
		for(int row = 0; row < n; row++){
			for (int col = row; col < n; col++){
				if (row==col)
					S[row][col] = 1;	//diagonal = 1
				else{
					if (align ((String)sequences.elementAt(row), (String)sequences.elementAt(col), thrd) >= 1){
						S[row][col] = 1;
						S[col][row] = 1;
					}
					else{
						S[row][col] = 0;
						S[col][row] = 0;
					}
				}
			}// end col
		}//end row
		
		
		/*output S for testing
		for(int row = 0; row < n; row++){
			for (int col = 0; col < n; col++){
				System.out.print(" " + S[row][col]);
			}
			System.out.print("\n");
		}
		*/
		
		// compute the score matrix
		for(int row = 0; row < n; row++){
			for (int col = 0; col < n; col++){
				if (S[row][col]==1)
					score[row]++;
			}
		}
		
		
		int index = -1;
		do {
			index = Utils.minIndex(score);
			if (score[index]< (n+1)){
				nr_set[nr_count++]= index;
				score[index] = n+1;
				for (int col = 0; col < n; col++){
					if (S[index][col]==1)
						score[col] = n+1;
				}				
			}
			else
				break;				
		} while (true);
		
		Writer w = new BufferedWriter (new FileWriter (outFile));
		
		for (int i = 1; i <= nr_count; i++){
			w.write(">" + i + "\n");
			w.write((String)sequences.elementAt(nr_set[i-1])+"\n");
		}
		w.close();
		System.out.println("size of " + dataFile + " is " + n);
		System.out.println("size of nr set is " + nr_count);
	}
	
	public Vector loadSequences (String dataFile) throws Exception {
		Vector seqs = new Vector();
		
		File content1 = new File (dataFile);
		InputStreamReader is1 =  new InputStreamReader (new FileInputStream (content1));
		BufferedReader reader1 = new BufferedReader (is1);
		String line = null;
		String s = null;		
		while((line=reader1.readLine())!= null){
			if (line.startsWith(">")){
				if (s == null)	//first seq
					s = new String();
				else{
					seqs.add(s);
					s = new String();
				}
			}
			else{
				s += line;
			}
		}
		seqs.add(s);
		
		return seqs;
	}
}
