/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    WeightRedundancy.java
 *    Copyright (C) 2008 Yasser EL-Manzalawy (yasser at cs.iastate.edu)
 *
 */


/*
 *    The input is a dataset of peptides (in fasta format).
 *    The output is the weight assigned to each peptide (see [1] for more details).
 *    Two peptides are similar if they share at least <cutoff>% value or they share a 9-mer subpeptide.
 *    This programs required the installation of EMBOSSWin package.
 */
/*
 * This programs has been used in [1] to derive WUPDS out of UPDS datasets.
 * [1] EL-Manzalawy, Y., Dobbs, D., Honavar, V., 2008. On evaluating MHC-II binding peptide prediction methods. Submitted to PLoS ONE.
 */

package evalscripts;

import java.io.*;
import java.util.*;
import weka.core.*;

public class WeightRedundancy {

	public static void main(String[] args) throws Exception{
		
		if (args.length < 3){
			System.out.println("Usage: WeightRedundancy <inFile> <outFile> <cutoff>");
			System.out.println("inFile and outFile are in fasta format");
			System.out.println("cutoff in percents (e.g., 0.8)");
			System.exit(0);
		}
		
		String inFile = args[0];
		String outFile = args[1];
		double thrd = Double.parseDouble(args[2]);

		WeightRedundancy wr = new WeightRedundancy();
		wr.getWeightedDataset(inFile, outFile, thrd);		
	}
	/***
	 * Aligns two sequences and return the similarity (e.g. 0.7)
	 * @param seq1
	 * @param seq2
	 * @return
	 * @throws Exception
	 */
	public double align(String seq1, String seq2) throws Exception{
		String tmp_dir = System.getProperty("java.io.tmpdir");	//get system tmp directory
		
		double id = 0;
		// write each sequence into a fasta file
		String file1 = tmp_dir+"s1.fasta";
		String file2 = tmp_dir+"s2.fasta";
		String outfile = tmp_dir+"algn.fasta";
		
		Writer w = new BufferedWriter (new FileWriter (file1));
		w.write(">s1 \n");
		w.write(seq1);
		w.close();
		
		w = new BufferedWriter (new FileWriter (file2));
		w.write(">s2 \n");
		w.write(seq2);
		w.close();
		
		// align
		try{
			Process p = null;			
			p = Runtime.getRuntime().exec("C:/EMBOSSwin/needle "+ file1 +" "+ file2+" -gapopen 10 -gapextend 0.5 -datafile EBLOSUM62 -outfile "+ tmp_dir+"algn.fasta" );
			int exitVal = p.waitFor();
		}catch (Exception e){
			System.out.println("align: Error aligning two sequences");
			e.printStackTrace();
	        System.out.println(e.getMessage());
		}
		//parse output file
		File content1 = new File (outfile);
		InputStreamReader is1 =  new InputStreamReader (new FileInputStream (content1));
		BufferedReader reader1 = new BufferedReader (is1);
		String line = null;
		while((line=reader1.readLine())!= null){			
			if(line.indexOf("Similarity:") > 0)				
				break;
		}
		id = Double.valueOf(line.substring(line.indexOf("(")+1, line.indexOf(")")-1)).doubleValue();
		return (id/100);
	}
	
	/***
	 * Two peptides will be similar if they share nonamer 
	 * @param seq1
	 * @param seq2
	 * @return 1 -> similar , otherwise 0
	 * @throws Exception
	 */
	public double align9 (String seq1, String seq2) throws Exception{
		int length = 9;
		if (seq1.length() <= seq2.length()){
			int len = seq1.length();
			for (int start = 0; start <= (len-length); start++){				
				int end = start + length;
				String seq = seq1.substring(start,end);
				if (seq2.indexOf(seq)>= 0)
					return 1;
			}			
		}
		else{
			int len = seq2.length();
			for (int start = 0; start <= (len-length); start++){				
				int end = start + length;
				String seq = seq2.substring(start,end);
				if (seq1.indexOf(seq)>= 0)
					return 1;
			}			
		}		
		return 0;
	}
	
	/**
	 * returns true if seq2 and seq2 are similar (e.g., share 9-mer subsequence or their sequence similarity >= thrd)
	 * @param seq1
	 * @param seq2
	 * @param thrd
	 * @return
	 * @throws Exception
	 */
	
	public boolean isSimilar(String seq1, String seq2, double thrd) throws Exception{
		if (align9(seq1, seq2)==1)
			return true;
		if (align(seq1, seq2)>= thrd)
			return true;
		return false;
	}

	/*****
	 * 
	 * @param dataFile sequences in fasta format
	 * @throws Exception
	 */
	public void getWeightedDataset (String dataFile, String outFile, double thrd) throws Exception {
		Vector sequences = loadSequences (dataFile);
		int n = sequences.size();
		int[][] S = new int [n][n];		//similarity matrix
		double[] score = new double [n];		//score[i] = no of sequences that sequence i is similar to them
		double[] weights = new double[n];		// weight of each sequence
		
		// init S matrix
		for(int row = 0; row < n; row++){
			for (int col = 0; col <= row; col++){
				if (row==col)
					S[row][col] = 0;	//diagonal = 0
				else{
					if (isSimilar((String)sequences.elementAt(row), (String)sequences.elementAt(col), thrd)){
						S[row][col] = 1;						
					}
					else{
						S[row][col] = 0;						
					}
				}
			}// end col
		}//end row

		// compute the score matrix
		for(int row = 0; row < n; row++){
			for (int col = 0; col < n; col++){
				if (S[row][col]==1)
					score[row]++;
			}
		}
		
		
		int index = -1;
		do {
			index = Utils.maxIndex(score);
			if (score[index]== -1)	//done
				break;
			else if (score[index]== 0){		// unique peptide
				score[index] = -1;
				weights[index]= 1;
			}
			else{				
				Vector positions = new Vector();	// vector of Integer objects
				score[index] = -1;
				positions.add(new Integer(index));
				
				for (int col = 0; col < index; col++){
					if ((S[index][col]==1)&&(score[col]!= -1)){
						//weights[col] = weight;
						score[col] = -1;
						positions.add(new Integer(col));
					}
				}				
				double weight = positions.size();
				weight = 1/weight;
				for (int p = 0; p < positions.size(); p++){
					weights[((Integer) positions.elementAt(p)).intValue()] = weight;
				}
			}
		} while (true);
		
		// write the weights
		Writer w = new BufferedWriter (new FileWriter (outFile));		
		for (int i = 0; i < n ; i++){			
			w.write(weights[i] +"\n");
		}
		w.close();
	}
	
	/**
	 * Loads peptides in fasta format
	 * @param dataFile fasta input file
	 * @return Vector of peptide sequences
	 * @throws Exception
	 */
	public Vector loadSequences (String dataFile) throws Exception {
		Vector seqs = new Vector();
		
		File content1 = new File (dataFile);
		InputStreamReader is1 =  new InputStreamReader (new FileInputStream (content1));
		BufferedReader reader1 = new BufferedReader (is1);
		String line = null;
		String s = null;		
		while((line=reader1.readLine())!= null){
			if (line.startsWith(">")){
				if (s == null)	//first seq
					s = new String();
				else{
					seqs.add(s);
					s = new String();
				}
			}
			else{
				s += line.toUpperCase().trim();
			}
		}
		seqs.add(s);
		
		return seqs;
	}

	
}
